void MQTT_init() {
  // not connected so first connect
  if (mqttclient.connect(mqtt_id)) {
    // we have a working connection

    #if USE_PCDIMMER == 1
      // subscribe to the desired topics
      mqttclient.subscribe(mqtt_topic_set_scene);
      mqttclient.subscribe(mqtt_topic_set_dimmtime);

      // register individual channels
      for (int i=0; i<lastDmxChannel; i++) {
        mqttclient.subscribe(String(mqtt_topic_set_channel + String(i+1)).c_str());
      }
    #endif

    for (int i=0; i<(lastVeluxChannel/2); i++) {
      mqttclient.subscribe(String(mqtt_topic_set_velux + String(i+1)).c_str());
    }
    // TODO: add more subscriptions here
  }
}

void MQTT_processMSG(char* topic, float value) {
  #if USE_PCDIMMER == 1
    if (strcmp(topic, mqtt_topic_set_scene) == 0) {
      // call the desired scene
      for (uint8_t ch=0; ch<lastDmxChannel; ch++) {
        if (scenes[(int)value][ch]>=0) {
          // channel is above -1 so it is active
          pcdimmerSetChannel(ch, scenes[(int)value][ch], dimmTime);
        }
      }
    }else if (strcmp(topic, mqtt_topic_set_dimmtime) == 0) {
      dimmTime = value;
    }else{
    // check for one of the channels
    for (uint8_t ch=0; ch<lastDmxChannel; ch++) {
      if (strcmp(topic, String(mqtt_topic_set_channel + String(ch+1)).c_str()) == 0) {
        pcdimmerSetChannel(ch, value, dimmTime);
      }
    }
  #else
  {
  #endif
    for (uint8_t ch=0; ch<(lastVeluxChannel/2); ch++) {
      if (strcmp(topic, String(mqtt_topic_set_velux + String(ch+1)).c_str()) == 0) {
        veluxShutter(ch, value);
      }
    }
  }
}

void MQTT_loop() {
  if (!mqttclient.connected()) {
    if (mqttReconnectTimeout == 0) {
      mqttReconnectTimeout = 30; // allow reconnecting not before 30 seconds
      MQTT_init();
    }
  }else{
    // connected to MQTT-server
    mqttReconnectTimeout = 0; // allow reconnection as soon as possible
    
    mqttclient.loop(); // process incoming messages
  }
}

void MQTT_publish() {
  #if USE_DALLAS == 1
    mqttclient.publish(mqtt_topic_temperature, String(temperature).c_str());
  #endif
  #if USE_DHT11 == 1
    if (dht_sensor.measure(&temperature, &humidity)) {
      mqttclient.publish(mqtt_topic_temperature, String(temperature).c_str());
      mqttclient.publish(mqtt_topic_humidity, String(humidity).c_str());
    }
  #endif
  #if USE_PCDIMMER == 1
    for (uint8_t ch=0; ch<lastDmxChannel; ch++) {
      mqttclient.publish(String(mqtt_topic_state_channel + String(ch+1)).c_str(), String(currentValue[ch]).c_str());
    }
  #endif
}

void MQTT_callback(char* topic, byte* payload, unsigned int length) {
  payload[length] = '\0'; // null-terminate byte-array

  // process received message
  MQTT_processMSG(topic, String((char*)payload).toFloat());
}