/*
  Control PC_DIMMER RS232-dimmerpacks
  Send and read PC_DIMMER control-frames
*/

#if USE_PCDIMMER == 1
  // send PC_DIMMER RS232-control-frames
  void pcdimmerSetChannel(uint8_t channel, uint8_t endvalue, uint32_t time) {
    if (channel < 0) {
      return;
    }
    if (channel >= lastDmxChannel) {
      return;
    }

    float temp;
    float t;
    uint8_t stp;
    uint32_t ftime;
    int32_t nsteps;

    t = time / 10;
    nsteps = currentValue[channel] - endvalue;
    nsteps = abs(nsteps);

    if ((nsteps != 0) && (t != 0)) {
      do {
        stp += 1;
        ftime = ((t * stp) / nsteps);
        temp = abs(((t - ((ftime * nsteps) / stp)) * 100) / t);
      } while((temp > 15) && (stp < 4));
    }else{
      ftime = 0;
      stp += 1;
    }

    uint8_t uartOutputBuffer[6];
    uartOutputBuffer[0] = channel + 128;
    uartOutputBuffer[1] = stp - 1;

    uartOutputBuffer[4] = ftime & 0x7F;
    uartOutputBuffer[3] = (ftime >> 7) & 0x7F;
    uartOutputBuffer[2] = (ftime >> 14) & 0x7F;

    uartOutputBuffer[5] = ((255 - endvalue) / 2) & 0x7F;

    Serial.write(uartOutputBuffer, 6);

    // set the internal value-storage to endvalue for MQTT-publishing
    // TODO: implement dimmerkernel and show correct value instead of final value
    currentValue[channel] = endvalue;
  }

  // receive PC_DIMMER RS232-control-frames
  void pcdimmerReadUart(uint8_t newData) {
    uartInputBuffer[uartCounter] = newData; // put input-character here
    uartCounter += 1;

    if (uartInputBuffer[0] < 128) {
      // wrong data
      uartCounter = 0;
    }else{
      // valid data in first array-element
      if (uartCounter >= 6) {
        uartCounter = 0;

        // validate last array-element
        if (uartInputBuffer[5] < 128) {
          // valid data -> use this data
          uint8_t channel = uartInputBuffer[0] - 128;
          uint8_t value = (127 - uartInputBuffer[5]) * 2;

          if (channel < lastDmxChannel) {
            currentValue[channel] = value;
          }
        }
      }
    }
  }
#endif