/*
  Control Velux KLF200 via GPIOs

  Each shutter-device has two inputs on the KLF200:
  - shutter 1: input 1+2
  - shutter 2: input 3+4
  - shutter 3: input 5+6
  - shutter 4: input 7+8
  - shutter 5: input 9+10

  Input 1 = Shutter up
  Input 2 = Shutter down
  Input 1+2 = Shutter stop

  Signals below 1.5s let the shutter run until the end is reached.
  Signals above 1.5s let the shutter run until the signal stops
*/
void veluxShutter(uint8_t channel, int8_t state) {
  if (state > 0) {
    // move up = enable KLF channel 1 for 0.7s
    veluxSetChannel(1 + channel * 2, true);
    shutterDisableCounter[channel * 2] = 7; // wait 0.7s
  }else if (state == 0) {
    // stop = enable KLF channel 1 and 2 for 0.7s
    veluxSetChannel(1 + channel * 2, true);
    veluxSetChannel(1 + channel * 2 + 1, true);
    shutterDisableCounter[channel * 2] = 7; // wait 0.7s
    shutterDisableCounter[channel * 2 + 1] = 7; // wait 0.7s
  }else if (state < 0) {
    // move down = enable KLF channel 2 for 0.7s
    veluxSetChannel(1 + channel * 2 + 1, true);
    shutterDisableCounter[channel * 2 + 1] = 7; // wait 0.7s
  }
}

void veluxSetChannel(uint8_t KLF_channel, bool state) {
  // KLF200 has pull-ups to 3V at the inputs. To enable
  // an input we have to turn-off the output of the ESP
  // so we have to invert the logic here
  
  switch(KLF_channel) {
    case 1:
      digitalWrite(D0, !state);
      break;
    case 2:
      digitalWrite(D1, !state);
      break;
    case 3:
      digitalWrite(D2, !state);
      break;
    case 4:
      digitalWrite(D3, !state);
      break;
    case 5:
      digitalWrite(D4, !state);
      break;
    case 6:
      digitalWrite(D5, !state);
      break;
    #if lastVeluxChannel >= 8
      case 7:
        digitalWrite(D6, !state);
        break;
      case 8:
        digitalWrite(D7, !state);
        break;
    #endif
    #if lastVeluxChannel == 10
      case 9:
        digitalWrite(3, !state); // RxD-Pin
        break;
      case 10:
        digitalWrite(1, !state); // TxD-Pin
        break;
    #endif
  }
}