#define UseStaticIP 1
#define lastDmxChannel 8
#define lastVeluxChannel 10 // supported options 6, 8 (of no DHT11/DS18B20), 10 (if no PC_DIMMER)
#define USE_DALLAS 0 // either DHT11 or DALLAS
#define USE_DHT11 0 // either DHT11 or DALLAS
#define USE_PCDIMMER 0

const char* versionstring = "v1.0.4 built on ";
const char compile_date[] = __DATE__ " " __TIME__;
const char* host = "vlx2mqtt";

#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <ESP8266HTTPUpdateServer.h>
#include <ESP8266WebServer.h>
#include <Ticker.h>
#include <PubSubClient.h>

#if USE_DALLAS == 1
  #define ONE_WIRE_BUS D6
  #include <OneWire.h> 
  #include <DallasTemperature.h>
  #include <NonBlockingDallas.h>
#endif
#if USE_DHT11 == 1
  #define DHT_SENSOR_PIN D6
  #define DHT_SENSOR_TYPE DHT_TYPE_11
  #include "DHT_Async.h"
#endif

// setup WiFi
const char* ssid = "MyWiFiSSID";
const char* password = "myWiFiPassword";
#if UseStaticIP == 1
  IPAddress local_IP(192, 168, 0, 42); // enter your desired static IP-Address
  IPAddress gateway(192, 168, 0, 1);
  IPAddress subnet(255, 255, 255, 0);
  IPAddress primaryDNS(192, 168, 0, 1);  //optional
  IPAddress secondaryDNS(1, 1, 1, 1);    //optional
#endif

// setup MQTT
#define mqtt_id "vlx2mqtt"
#define mqtt_server "192.168.0.1" // enter the IP-Address your MQTT-Broker
#define mqtt_serverport 1883

#if USE_DALLAS == 1 || USE_DHT11 == 1
  const char* mqtt_topic_temperature = "vlx2mqtt/temperature";
#endif
#if USE_DHT11 == 1
  const char* mqtt_topic_humidity = "vlx2mqtt/humidity";
#endif
# if USE_PCDIMMER == 1
  const char* mqtt_topic_set_dimmtime = "vlx2mqtt/lights/dimmtime";
  const char* mqtt_topic_set_scene = "vlx2mqtt/lights/scene"; // vlx2mqtt/lights/scene
  const char* mqtt_topic_set_channel = "vlx2mqtt/lights/ch/"; // vlx2mqtt/lights/ch/1 ... vlx2mqtt/lights/ch/8
  const char* mqtt_topic_state_channel = "vlx2mqtt/lights/state/ch/"; // vlx2mqtt/lights/state/ch/1 ... vlx2mqtt/lights/state/ch/8
#endif
const char* mqtt_topic_set_velux = "vlx2mqtt/shutter/velux"; // vlx2mqtt/shutter/velux1 ... vlx2mqtt/shutter/velux5

WiFiEventHandler gotIpEventHandler, disconnectedEventHandler;
bool WiFiAvailable = false;
WiFiClient mqttnetworkclient; // MQTT-Client
PubSubClient mqttclient(mqttnetworkclient);
uint8_t mqttReconnectTimeout = 0;

ESP8266WebServer webserver(80); // HTTP-Server
ESP8266HTTPUpdateServer httpupdater; // WebUpdater
Ticker TimerSeconds;
Ticker Timer100ms;
#if lastVeluxChannel == 10
  uint8_t shutterDisableCounter[lastVeluxChannel] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
#elif lastVeluxChannel == 8
  uint8_t shutterDisableCounter[lastVeluxChannel] = {0, 0, 0, 0, 0, 0, 0, 0};
#else
  uint8_t shutterDisableCounter[lastVeluxChannel] = {0, 0, 0, 0, 0, 0};
#endif

#if USE_DALLAS == 1
  OneWire oneWire(ONE_WIRE_BUS);
  DallasTemperature dallasTemp(&oneWire);
  NonBlockingDallas temperatureSensors(&dallasTemp);
  float temperature;
#endif
#if USE_DHT11 == 1
  DHT_Async dht_sensor(DHT_SENSOR_PIN, DHT_SENSOR_TYPE);
  float temperature;
  float humidity;
#endif
#if USE_PCDIMMER == 1
  uint8_t currentValue[lastDmxChannel];
  uint32_t dimmTime = 3000;
  uint8_t uartCounter = 0;
  uint8_t uartInputBuffer[6];

  // an entry below 0 will be ignored from the scene-call-function
  int16_t scenes[6][lastDmxChannel] = {
  //   1    2    3    4    5    6    7    8
    {  0,   0,   0,   0,   0,  -1,   0,  -1}, // Blackout
    {  0,   0,   0, 255, 255,  -1, 255,  -1}, // Scene 1: Alles hell
    {  0,   0,   0,   0, 255,  -1,   0,  -1}, // Scene 2: nur Esstisch
    {  0,   0,   0,   0,   0,  -1, 255,  -1}, // Scene 3: nur Sofa
    {  0,   0,   0, 255,   0,  -1,   0,  -1}, // Scene 4: nur Regal
    {  0,   0, 255,  64,  64,  -1,  64,  -1}  // Scene 5: gemütlich
  };
#endif